//A program to write data to a pipe in one process and read from it in 
//another process.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>

//The buffer size.
#define BUF_SIZE 1000

//The read end of the pipe.
#define R_END 0

//The write end of the pipe.
#define W_END 1

int main(int argc, char ** argv)
{
  int fd[2];  //File descriptors for the read and write ends of the pipe
  pid_t pid;  //The process id

  char r_mesg[BUF_SIZE];  //Store the read message here

  //The message to write.
  char w_mesg[BUF_SIZE] = "1.0 2.0 3.0 4.0 5.0 -6.0 7.0 8.0 9.0 10.0 11.0 12.0 13.0 14.0 15.0 16.0 17.0 18.0 19.0 20.0";

  if(pipe(fd) == -1)
  {
    fprintf(stderr,"Failed to create pipe.\n");
    exit(-1);
  }

  //Fork child process
  pid = fork();

  //Handle for error.
  if(pid < 0)
  {
    fprintf(stderr, "Failed to fork child process.\n");
    exit(-1);
  }

  //Handle the parent process.
  if(pid > 0)
  {
    //Close unused end of the pipe.
    close(fd[R_END]);

    //Write data to the pipe.
    write(fd[W_END], w_mesg, strlen(w_mesg)+1);

    //Close the write end of the pipe.
    close(fd[W_END]);
  }
  //Handle the child process.
  else
  {
    //Close the unused end of the pipe
    close(fd[W_END]);

    //Read data from the pipe
    read(fd[R_END], r_mesg, BUF_SIZE);

    //Use the string tokenizer to parse the message.
    double arr[50];
    int count = 0;
    int i;

    char * tok = strtok(r_mesg, " ");

    while (tok != NULL)
    {
      arr[count] = atof(tok);
      tok = strtok(NULL, " ");
      count++;
    }

    printf("Printing data from pipe...\n");

    for(i = 0; i < count; i++)
      if(arr[i] >= 0.0)
        printf("%lf is positive\n", arr[i]);
      else
        printf("%lf is negative\n", arr[i]);

    //Close the read end of the pipe.
    close(fd[R_END]);

  }

  return 0;
}
